@@ -24,26 +24,13 @@
 """Converting MySQL and Python types
 """
 
-import struct
 import datetime
 import time
 from decimal import Decimal
 
-from mysql.connector.constants import FieldType, FieldFlag, CharacterSet
-
-
-class HexLiteral(str):
-
-    """Class holding MySQL hex literals"""
-    def __new__(cls, str_, charset='utf8'):
-        hexed = ["{0:x}".format(ord(i)) for i in str_.encode(charset)]
-        obj = str.__new__(cls, ''.join(hexed))
-        obj.charset = charset
-        obj.original = str_
-        return obj
-
-    def __str__(self):
-        return '0x' + self
+from .constants import FieldType, FieldFlag, CharacterSet
+from .catch23 import PY2, NUMERIC_TYPES, struct_unpack
+from .custom_types import HexLiteral
 
 
 class MySQLConverterBase(object):
@@ -52,6 +39,7 @@ class MySQLConverterBase(object):
     All class dealing with converting to and from MySQL data types must
     be a subclass of this class.
     """
+
     def __init__(self, charset='utf8', use_unicode=True):
         self.python_types = None
         self.mysql_types = None
@@ -95,6 +83,7 @@ class MySQLConverterBase(object):
 
 class MySQLConverter(MySQLConverterBase):
     """Default conversion class for MySQL Connector/Python.
+
      o escape method: for escaping values send to MySQL
      o quoting method: for quoting values send to MySQL in statements
      o conversion mapping: maps Python and MySQL data types to
@@ -105,6 +94,7 @@ class MySQLConverter(MySQLConverterBase):
     cnx.connect(converter_class=CustomMySQLConverterClass).
 
     """
+
     def __init__(self, charset=None, use_unicode=True):
         MySQLConverterBase.__init__(self, charset, use_unicode)
         self._cache_field_types = {}
@@ -119,39 +109,51 @@ class MySQLConverter(MySQLConverterBase):
         """
         if value is None:
             return value
-        elif isinstance(value, (int, float, long, Decimal, HexLiteral)):
+        elif isinstance(value, NUMERIC_TYPES):
             return value
-        res = value
-        res = res.replace('\\', '\\\\')
-        res = res.replace('\n', '\\n')
-        res = res.replace('\r', '\\r')
-        res = res.replace('\047', '\134\047')  # single quotes
-        res = res.replace('\042', '\134\042')  # double quotes
-        res = res.replace('\032', '\134\032')  # for Win32
-        return res
+        if isinstance(value, (bytes, bytearray)):
+            value = value.replace(b'\\', b'\\\\')
+            value = value.replace(b'\n', b'\\n')
+            value = value.replace(b'\r', b'\\r')
+            value = value.replace(b'\047', b'\134\047')  # single quotes
+            value = value.replace(b'\042', b'\134\042')  # double quotes
+            value = value.replace(b'\032', b'\134\032')  # for Win32
+        else:
+            value = value.replace('\\', '\\\\')
+            value = value.replace('\n', '\\n')
+            value = value.replace('\r', '\\r')
+            value = value.replace('\047', '\134\047')  # single quotes
+            value = value.replace('\042', '\134\042')  # double quotes
+            value = value.replace('\032', '\134\032')  # for Win32
+        return value
 
     def quote(self, buf):
         """
         Quote the parameters for commands. General rules:
-          o numbers are returns as str type (because operation expect it)
-          o None is returned as str('NULL')
-          o String are quoted with single quotes '<string>'
+          o numbers are returns as bytes using ascii codec
+          o None is returned as bytearray(b'NULL')
+          o Everything else is single quoted '<buf>'
 
-        Returns a string.
+        Returns a bytearray object.
         """
-        if isinstance(buf, (int, float, long, Decimal, HexLiteral)):
-            return str(buf)
+        if isinstance(buf, NUMERIC_TYPES):
+            if PY2:
+                if isinstance(buf, float):
+                    return repr(buf)
+                else:
+                    return str(buf)
+            else:
+                return str(buf).encode('ascii')
         elif isinstance(buf, type(None)):
-            return "NULL"
+            return bytearray(b"NULL")
         else:
-            # Anything else would be a string
-            return "'%s'" % buf
+            return bytearray(b"'" + buf + b"'")
 
     def to_mysql(self, value):
         """Convert Python data type to MySQL"""
         type_name = value.__class__.__name__.lower()
         try:
-            return getattr(self, "_%s_to_mysql" % str(type_name))(value)
+            return getattr(self, "_{0}_to_mysql".format(type_name))(value)
         except AttributeError:
             raise TypeError("Python '{0}' cannot be converted to a "
                             "MySQL type".format(type_name))
@@ -161,8 +163,8 @@ class MySQLConverter(MySQLConverterBase):
         return int(value)
 
     def _long_to_mysql(self, value):
-        """Convert value to long"""
-        return long(value)
+        """Convert value to int"""
+        return int(value)
 
     def _float_to_mysql(self, value):
         """Convert value to float"""
@@ -170,19 +172,26 @@ class MySQLConverter(MySQLConverterBase):
 
     def _str_to_mysql(self, value):
         """Convert value to string"""
-        return str(value)
+        if PY2:
+            return str(value)
+        return self._unicode_to_mysql(value)
 
     def _unicode_to_mysql(self, value):
-        """
-        Encodes value, a Python unicode string, to whatever the
-        character set for this converter is set too.
-        """
+        """Convert unicode"""
         encoded = value.encode(self.charset)
         if self.charset_id in CharacterSet.slash_charsets:
-            if '\x5c' in encoded:
+            if b'\x5c' in encoded:
                 return HexLiteral(value, self.charset)
         return encoded
 
+    def _bytes_to_mysql(self, value):
+        """Convert value to bytes"""
+        return value
+
+    def _bytearray_to_mysql(self, value):
+        """Convert value to bytes"""
+        return str(value)
+
     def _bool_to_mysql(self, value):
         """Convert value to boolean"""
         if value:
@@ -207,16 +216,19 @@ class MySQLConverter(MySQLConverterBase):
 
         If the instance isn't a datetime.datetime type, it return None.
 
-        Returns a string.
+        Returns a bytes.
         """
         if value.microsecond:
-            return '%d-%02d-%02d %02d:%02d:%02d.%06d' % (
+            fmt = '{0:d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}.{6:06d}'
+            return fmt.format(
                 value.year, value.month, value.day,
                 value.hour, value.minute, value.second,
-                value.microsecond)
-        return '%d-%02d-%02d %02d:%02d:%02d' % (
+                value.microsecond).encode('ascii')
+
+        fmt = '{0:d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}'
+        return fmt.format(
             value.year, value.month, value.day,
-            value.hour, value.minute, value.second)
+            value.hour, value.minute, value.second).encode('ascii')
 
     def _date_to_mysql(self, value):
         """
@@ -225,9 +237,10 @@ class MySQLConverter(MySQLConverterBase):
 
         If the instance isn't a datetime.date type, it return None.
 
-        Returns a string.
+        Returns a bytes.
         """
-        return '%d-%02d-%02d' % (value.year, value.month, value.day)
+        return '{0:d}-{1:02d}-{2:02d}'.format(value.year, value.month,
+                                              value.day).encode('ascii')
 
     def _time_to_mysql(self, value):
         """
@@ -236,11 +249,11 @@ class MySQLConverter(MySQLConverterBase):
 
         If the instance isn't a datetime.time type, it return None.
 
-        Returns a string or None when not valid.
+        Returns a bytes.
         """
         if value.microsecond:
-            return value.strftime('%H:%M:%S.%%06d') % value.microsecond
-        return value.strftime('%H:%M:%S')
+            return value.strftime('%H:%M:%S.%f').encode('ascii')
+        return value.strftime('%H:%M:%S').encode('ascii')
 
     def _struct_time_to_mysql(self, value):
         """
@@ -248,34 +261,54 @@ class MySQLConverter(MySQLConverterBase):
         for MySQL.
         The returned string has format: %Y-%m-%d %H:%M:%S
 
-        Returns a string or None when not valid.
+        Returns a bytes or None when not valid.
         """
-        return time.strftime('%Y-%m-%d %H:%M:%S', value)
+        return time.strftime('%Y-%m-%d %H:%M:%S', value).encode('ascii')
 
     def _timedelta_to_mysql(self, value):
         """
         Converts a timedelta instance to a string suitable for MySQL.
         The returned string has format: %H:%M:%S
 
-        Returns a string.
+        Returns a bytes.
         """
-        (hours, remainder) = divmod(value.seconds, 3600)
-        (mins, secs) = divmod(remainder, 60)
-        hours = hours + (value.days * 24)
+        seconds = abs(value.days * 86400 + value.seconds)
+
         if value.microseconds:
-            return '%02d:%02d:%02d.%06d' % (hours, mins, secs,
-                                            value.microseconds)
-        return '%02d:%02d:%02d' % (hours, mins, secs)
+            fmt = '{0:02d}:{1:02d}:{2:02d}.{3:06d}'
+            if value.days < 0:
+                mcs = 1000000 - value.microseconds
+                seconds -= 1
+            else:
+                mcs = value.microseconds
+        else:
+            fmt = '{0:02d}:{1:02d}:{2:02d}'
+
+        if value.days < 0:
+            fmt = '-' + fmt
+
+        (hours, remainder) = divmod(seconds, 3600)
+        (mins, secs) = divmod(remainder, 60)
+
+        if value.microseconds:
+            result = fmt.format(hours, mins, secs, mcs)
+        else:
+            result = fmt.format(hours, mins, secs)
+
+        if PY2:
+            return result
+        else:
+            return result.encode('ascii')
 
     def _decimal_to_mysql(self, value):
         """
         Converts a decimal.Decimal instance to a string suitable for
         MySQL.
 
-        Returns a string or None when not valid.
+        Returns a bytes or None when not valid.
         """
         if isinstance(value, Decimal):
-            return str(value)
+            return str(value).encode('ascii')
 
         return None
 
@@ -287,7 +320,7 @@ class MySQLConverter(MySQLConverterBase):
 
         Returns a mixed value.
         """
-        if value == '\x00' and flddsc[1] != FieldType.BIT:
+        if value == 0 and flddsc[1] != FieldType.BIT:  # \x00
             # Don't go further when we hit a NULL value
             return None
         if value is None:
@@ -307,7 +340,10 @@ class MySQLConverter(MySQLConverterBase):
             return self._cache_field_types[flddsc[1]](value, flddsc)
         except KeyError:
             # If one type is not defined, we just return the value as str
-            return str(value)
+            try:
+                return value.decode('utf-8')
+            except UnicodeDecodeError:
+                return value
         except ValueError as err:
             raise ValueError("%s (field %s)" % (err, flddsc[0]))
         except TypeError as err:
@@ -315,11 +351,58 @@ class MySQLConverter(MySQLConverterBase):
         except:
             raise
 
+    def row_to_python(self, row, fields):
+        """Convert a MySQL text result row to Python types
+
+        The row argument is a sequence containing text result returned
+        by a MySQL server. Each value of the row is converted to the
+        using the field type information in the fields argument.
+
+        Returns a tuple.
+        """
+        i = 0
+        result = [None]*len(fields)
+
+        if not self._cache_field_types:
+            self._cache_field_types = {}
+            for name, info in FieldType.desc.items():
+                try:
+                    self._cache_field_types[info[0]] = getattr(
+                        self, '_{0}_to_python'.format(name))
+                except AttributeError:
+                    # We ignore field types which has no method
+                    pass
+
+        for field in fields:
+            field_type = field[1]
+
+            if (row[i] == 0 and field_type != FieldType.BIT) or row[i] is None:
+                # Don't convert NULL value
+                i += 1
+                continue
+
+            try:
+                result[i] = self._cache_field_types[field_type](row[i], field)
+            except KeyError:
+                # If one type is not defined, we just return the value as str
+                try:
+                    result[i] = row[i].decode('utf-8')
+                except UnicodeDecodeError:
+                    result[i] = row[i]
+            except (ValueError, TypeError) as err:
+                err.message = "{0} (field {1})".format(str(err), field[0])
+                raise
+
+            i += 1
+
+        return tuple(result)
+
     def _FLOAT_to_python(self, value, desc=None):  # pylint: disable=C0103
         """
         Returns value as float type.
         """
         return float(value)
+
     _DOUBLE_to_python = _FLOAT_to_python
 
     def _INT_to_python(self, value, desc=None):  # pylint: disable=C0103
@@ -327,22 +410,20 @@ class MySQLConverter(MySQLConverterBase):
         Returns value as int type.
         """
         return int(value)
+
     _TINY_to_python = _INT_to_python
     _SHORT_to_python = _INT_to_python
     _INT24_to_python = _INT_to_python
-
-    def _LONG_to_python(self, value, desc=None):  # pylint: disable=C0103
-        """
-        Returns value as long type.
-        """
-        return int(value)
-    _LONGLONG_to_python = _LONG_to_python
+    _LONG_to_python = _INT_to_python
+    _LONGLONG_to_python = _INT_to_python
 
     def _DECIMAL_to_python(self, value, desc=None):  # pylint: disable=C0103
         """
         Returns value as a decimal.Decimal.
         """
-        return Decimal(value)
+        val = value.decode(self.charset)
+        return Decimal(val)
+
     _NEWDECIMAL_to_python = _DECIMAL_to_python
 
     def _str(self, value, desc=None):
@@ -355,18 +436,19 @@ class MySQLConverter(MySQLConverterBase):
         """Returns BIT columntype as integer"""
         int_val = value
         if len(int_val) < 8:
-            int_val = '\x00' * (8-len(int_val)) + int_val
-        return struct.unpack('>Q', int_val)[0]
+            int_val = b'\x00' * (8 - len(int_val)) + int_val
+        return struct_unpack('>Q', int_val)[0]
 
     def _DATE_to_python(self, value, dsc=None):  # pylint: disable=C0103
         """
         Returns DATE column type as datetime.date type.
         """
         try:
-            parts = value.split('-')
+            parts = value.split(b'-')
             return datetime.date(int(parts[0]), int(parts[1]), int(parts[2]))
         except ValueError:
             return None
+
     _NEWDATE_to_python = _DATE_to_python
 
     def _TIME_to_python(self, value, dsc=None):  # pylint: disable=C0103
@@ -375,18 +457,21 @@ class MySQLConverter(MySQLConverterBase):
         """
         time_val = None
         try:
-            (hms, mcs) = value.split('.')
-            mcs = int(mcs.ljust(6, '0'))
+            (hms, mcs) = value.split(b'.')
+            mcs = int(mcs.ljust(6, b'0'))
         except ValueError:
             hms = value
             mcs = 0
         try:
-            (hour, mins, sec) = [int(d) for d in hms.split(':')]
-            time_val = datetime.timedelta(hours=hour, minutes=mins, seconds=sec,
-                                    microseconds=mcs)
+            (hours, mins, secs) = [int(d) for d in hms.split(b':')]
+            if value[0] == 45 or value[0] == '-':  # if PY3 or PY2
+                mins, secs, mcs = -mins, -secs, -mcs
+            time_val = datetime.timedelta(hours=hours, minutes=mins,
+                                          seconds=secs, microseconds=mcs)
         except ValueError:
             raise ValueError(
-                "Could not convert %s to python datetime.timedelta" % value)
+                "Could not convert {0} to python datetime.timedelta".format(
+                    value))
         else:
             return time_val
 
@@ -396,20 +481,21 @@ class MySQLConverter(MySQLConverterBase):
         """
         datetime_val = None
         try:
-            (date_, time_) = value.split(' ')
+            (date_, time_) = value.split(b' ')
             if len(time_) > 8:
-                (hms, mcs) = time_.split('.')
-                mcs = int(mcs.ljust(6, '0'))
+                (hms, mcs) = time_.split(b'.')
+                mcs = int(mcs.ljust(6, b'0'))
             else:
                 hms = time_
                 mcs = 0
-            dtval = [int(value) for value in date_.split('-')] +\
-                 [int(value) for value in hms.split(':')] + [mcs,]
+            dtval = [int(i) for i in date_.split(b'-')] + \
+                    [int(i) for i in hms.split(b':')] + [mcs, ]
             datetime_val = datetime.datetime(*dtval)
         except ValueError:
             datetime_val = None
 
         return datetime_val
+
     _TIMESTAMP_to_python = _DATETIME_to_python
 
     def _YEAR_to_python(self, value, desc=None):  # pylint: disable=C0103
@@ -430,10 +516,11 @@ class MySQLConverter(MySQLConverterBase):
         Returns SET column type as a set.
         """
         set_type = None
+        val = value.decode(self.charset)
         try:
-            set_type = set(value.split(','))
+            set_type = set(val.split(','))
         except ValueError:
-            raise ValueError("Could not convert SET %s to a set." % value)
+            raise ValueError("Could not convert set %s to a sequence." % value)
         return set_type
 
     def _STRING_to_python(self, value, dsc=None):  # pylint: disable=C0103
@@ -450,22 +537,24 @@ class MySQLConverter(MySQLConverterBase):
             if dsc[7] & FieldFlag.BINARY:
                 return value
 
-        if self.use_unicode:
-            try:
-                return unicode(value, self.charset)
-            except:
-                raise
+        if isinstance(value, (bytes, bytearray)) and self.use_unicode:
+            return value.decode(self.charset)
+
+        return value
 
-        return str(value)
     _VAR_STRING_to_python = _STRING_to_python
 
     def _BLOB_to_python(self, value, dsc=None):  # pylint: disable=C0103
         """Convert BLOB data type to Python"""
         if dsc is not None:
             if dsc[7] & FieldFlag.BINARY:
-                return value
+                if PY2:
+                    return value
+                else:
+                    return bytes(value)
 
         return self._STRING_to_python(value, dsc)
+
     _LONG_BLOB_to_python = _BLOB_to_python
     _MEDIUM_BLOB_to_python = _BLOB_to_python
     _TINY_BLOB_to_python = _BLOB_to_python
