@@ -31,8 +31,10 @@ Servers are organized into groups which have unique names. This aims
 at defining administrative domains and easing management activities.
 """
 import threading
+import time
 import uuid as _uuid
 import logging
+import math
 import functools
 import re
 
@@ -93,9 +95,11 @@ class Group(_persistence.Persistable):
                     "(group_id VARCHAR(64) NOT NULL, "
                     "description VARCHAR(256), "
                     "master_uuid VARCHAR(40), "
-                    "master_defined TIMESTAMP(6) NULL, "
+                    "master_defined TIMESTAMP /*!50604 (6) */ NULL, "
                     "status BIT(1) NOT NULL, "
-                    "CONSTRAINT pk_group_id PRIMARY KEY (group_id))")
+                    "CONSTRAINT pk_group_id PRIMARY KEY (group_id)) "
+                    "DEFAULT CHARSET=utf8"
+    )
 
     #Create the table that stores the group replication relationship.
     CREATE_GROUP_REPLICATION = (
@@ -108,7 +112,9 @@ class Group(_persistence.Persistable):
                             "REFERENCES groups(group_id), "
                             "CONSTRAINT FOREIGN KEY(slave_group_id) "
                             "REFERENCES groups(group_id), "
-                            "INDEX idx_slave_group_id(slave_group_id))")
+                            "INDEX idx_slave_group_id(slave_group_id)) "
+                            "DEFAULT CHARSET=utf8"
+    )
 
     #SQL statement for inserting a new group into the table
     INSERT_GROUP = ("INSERT INTO groups(group_id, description, status) "
@@ -223,7 +229,8 @@ class Group(_persistence.Persistable):
         """
         ret = []
         rows = persister.exec_stmt(Group.QUERY_GROUP_REPLICATION_SLAVES,
-            {"fetch" : True, "params" : (self.__group_id,)})
+            {"params" : (self.__group_id,)}
+        )
         if not rows:
             return ret
 
@@ -245,7 +252,7 @@ class Group(_persistence.Persistable):
                           state store.
         """
         row = persister.exec_stmt(Group.QUERY_GROUP_REPLICATION_MASTER,
-            {"fetch" : True, "params" : (self.__group_id,)})
+            {"params" : (self.__group_id,)})
         if not row:
             return None
         return row[0][0]
@@ -396,6 +403,25 @@ class Group(_persistence.Persistable):
         """
         return self.__status
 
+    def kill_connections_on_servers(self):
+        """Kill all the threads on the groups servers.
+        """
+        server = None
+        server_kill_threads = []
+        while server in self.servers():
+            try:
+                server.connect()
+            except _errors.UuidError,  _errors.DatabaseError:
+                continue
+            if server.is_connected():
+                server_kill_threads.append(
+                    threading.Thread(
+                        target=server.kill_processes(server.processes())
+                    )
+                )
+        for thread in server_kill_threads:
+            thread.join()
+
     @status.setter
     def status(self, status, persister=None):
         """Set the group's status.
@@ -446,7 +472,7 @@ class Group(_persistence.Persistable):
 
         :param persister: Persister to persist the object to.
         """
-        return persister.exec_stmt(Group.QUERY_GROUPS)
+        return [ gid[0] for gid in persister.exec_stmt(Group.QUERY_GROUPS) ]
 
     @staticmethod
     def remove(group, persister=None):
@@ -472,9 +498,8 @@ class Group(_persistence.Persistable):
         """
         group = None
         cur = persister.exec_stmt(
-            Group.QUERY_GROUP, {"fetch" : False, "raw" : False,
-            "params" : (group_id,)}
-            )
+            Group.QUERY_GROUP, {"fetch" : False, "params" : (group_id, )}
+        )
         row = cur.fetchone()
         if row:
             group_id, description, master, master_defined, status = row
@@ -631,6 +656,7 @@ class MySQLServer(_persistence.Persistable):
                    receiving requests.
     :type weight: Float
     :param group_id: Group's id which the server belongs to.
+    :param row: Row with information on the server.
     """
     #SQL Statement for creating the table used to store details about the
     #server.
@@ -643,7 +669,9 @@ class MySQLServer(_persistence.Persistable):
         "weight FLOAT NOT NULL, "
         "group_id VARCHAR(64), "
         "CONSTRAINT pk_server_uuid PRIMARY KEY (server_uuid), "
-        "INDEX idx_group_id (group_id))"
+        "INDEX idx_group_id (group_id), "
+        "UNIQUE INDEX idx_server_address (server_address)) "
+        "DEFAULT CHARSET=utf8"
     )
 
     #Create the referential integrity constraint with the groups table
@@ -679,18 +707,24 @@ class MySQLServer(_persistence.Persistable):
     #SQL statement used for deleting the server identified by the server id.
     REMOVE_SERVER = ("DELETE FROM servers WHERE server_uuid = %s")
 
-    #SQL Statement to retrieve the server from the state_store.
-    QUERY_SERVER = (
+    #SQL Statement to retrieve the server from the state store.
+    QUERY_SERVER_BY_UUID = (
         "SELECT server_uuid, server_address, mode, status, weight, group_id "
         "FROM servers WHERE server_uuid = %s"
         )
 
-    #SQL Statement to retrieve a set of servers in a group from the state_store.
-    QUERY_SERVER_GROUP_ID = (
-        "SELECT server_uuid, server_address, mode, status, weight "
+    #SQL Statement to retrieve a set of servers in a group from the state store.
+    QUERY_SERVER_BY_GROUP_ID = (
+        "SELECT server_uuid, server_address, mode, status, weight, group_id "
         "FROM servers WHERE group_id = %s"
         )
 
+    #SQL Statement to retrieve a server from the state store.
+    QUERY_SERVER_BY_ADDRESS = (
+        "SELECT server_uuid, server_address, mode, status, weight, group_id "
+        "FROM servers WHERE server_address = %s"
+        )
+
     #SQL Statement to retrieve the servers belonging to a group.
     DUMP_SERVERS = (
         "SELECT server_uuid, group_id, server_address, mode, status, weight "
@@ -754,14 +788,24 @@ class MySQLServer(_persistence.Persistable):
 
     NO_USER_DATABASES = ["performance_schema", "information_schema", "mysql"]
 
-    def __init__(self, uuid, address, user=None, passwd=None, mode=DEFAULT_MODE,
-                 status=DEFAULT_STATUS, weight=DEFAULT_WEIGHT, group_id=None):
+    def __init__(self, uuid=None, address=None, user=None, passwd=None,
+                 mode=DEFAULT_MODE, status=DEFAULT_STATUS,
+                 weight=DEFAULT_WEIGHT, group_id=None, row=None):
         """Constructor for MySQLServer.
         """
         super(MySQLServer, self).__init__()
+
+        if row is not None:
+            assert(uuid is None and address is None)
+            uuid, address, idx_mode, idx_status, weight, group_id = row
+            mode = MySQLServer.get_mode(idx_mode)
+            status = MySQLServer.get_status(idx_status)
+            uuid = _uuid.UUID(uuid)
+
         assert(isinstance(uuid, _uuid.UUID))
         assert(mode in MySQLServer.SERVER_MODE)
         assert(status in MySQLServer.SERVER_STATUS)
+
         self.__cnx = None
         self.__uuid = uuid
         self.__address = address
@@ -801,13 +845,13 @@ class MySQLServer(_persistence.Persistable):
         passwd = passwd or MySQLServer.PASSWD
         cnx = _server_utils.create_mysql_connection(
             user=user, passwd=passwd, host=host, port=port, autocommit=True,
-            use_unicode=False, connection_timeout=connection_timeout
+            connection_timeout=connection_timeout
         )
 
         try:
             row = _server_utils.exec_mysql_stmt(cnx,
                 "SELECT @@GLOBAL.SERVER_UUID")
-            server_uuid = row[0][0]
+            server_uuid = str(row[0][0])
         finally:
             _server_utils.destroy_mysql_connection(cnx)
 
@@ -830,7 +874,7 @@ class MySQLServer(_persistence.Persistable):
             _server_utils.MYSQL_DEFAULT_PORT)
 
         return _server_utils.create_mysql_connection(
-            autocommit=True, use_unicode=False, host=host, port=port,
+            autocommit=True, host=host, port=port,
             user=self.user, passwd=self.passwd,
             connection_timeout=connection_timeout
         )
@@ -896,14 +940,6 @@ class MySQLServer(_persistence.Persistable):
             self.__gtid_enabled = None
             self.__binlog_enabled = None
 
-    def user_databases(self):
-        """Return user databases.
-        """
-        exc_dbs = MySQLServer.NO_USER_DATABASES
-        dbs = self.exec_stmt("SHOW DATABASES")
-        dbs = [db[0] for db in dbs if db[0] not in exc_dbs]
-        return dbs
-
     def has_privileges(self, required_privileges, level=None):
         """Check whether the current user has the required privileges.
 
@@ -1167,17 +1203,11 @@ class MySQLServer(_persistence.Persistable):
                           state store.
         """
         ret = []
-        rows = persister.exec_stmt(MySQLServer.QUERY_SERVER_GROUP_ID,
-            {"raw" : False, "params" : (group_id, )}
+        rows = persister.exec_stmt(MySQLServer.QUERY_SERVER_BY_GROUP_ID,
+            {"params" : (group_id, )}
         )
         for row in rows:
-            uuid, address, idx_mode, idx_status, weight = row
-            mode = MySQLServer.get_mode(idx_mode)
-            status = MySQLServer.get_status(idx_status)
-            server = MySQLServer(
-                _uuid.UUID(uuid), address=address, mode=mode, status=status,
-                weight=weight, group_id=group_id
-            )
+            server = MySQLServer(row=row)
             ret.append(server)
         return ret
 
@@ -1226,7 +1256,6 @@ class MySQLServer(_persistence.Persistable):
             "@@GLOBAL.GTID_PURGED as GTID_PURGED, "
             "@@GLOBAL.GTID_OWNED as GTID_OWNED"
         )
-
         return self.exec_stmt(query_str, {"columns" : True})
 
     def has_storage_engine(self, target):
@@ -1304,7 +1333,7 @@ class MySQLServer(_persistence.Persistable):
         assert(context in MySQLServer.CONTEXTS)
         ret = self.exec_stmt("SELECT @@%s.%s as %s" %
                              (context, variable, variable))
-        return ret[0][0]
+        return str(ret[0][0])
 
     def set_variable(self, variable, value, context=None):
         """Execute the SET command for the client and return a result set.
@@ -1315,6 +1344,68 @@ class MySQLServer(_persistence.Persistable):
         return self.exec_stmt("SET @@%s.%s = %s" %
                               (context, variable, value))
 
+    def processes(self, system_user=False, current_connection=False):
+        """Return a list of processes running on the MySQL Server.
+        Skip the system processes that might be running on the server.
+        Skip also the ID of the current connection used to retrieve the list
+        of IDs.
+
+        :param system_user: Set to True if the system users needs to be
+                            included in the result.
+        :param current_connection: Set to True if the current connection needs
+                                    to be included in the result.
+        :return: The list of process IDs of processes running on the server.
+        """
+        SELECT_IDs = "SELECT ID FROM INFORMATION_SCHEMA.PROCESSLIST"
+        if not system_user or not current_connection:
+            SELECT_IDs += " WHERE"
+        if not system_user:
+            SELECT_IDs += " User != 'system user' AND User != 'event_scheduler'"
+        if not current_connection:
+            if not system_user:
+                SELECT_IDs += " AND"
+            SELECT_IDs += " ID != CONNECTION_ID()"
+
+        #User != 'system user' skips over any replication threads that might be
+        #running in the system.
+        proc_ids = []
+        proc_ids_rows = self.exec_stmt(SELECT_IDs)
+        for row in proc_ids_rows:
+            proc_ids.append(row[0])
+        return proc_ids
+
+    def kill_processes(self,  proc_ids):
+        """Kill the list of processes supplied as an argument.
+
+        :param proc_ids: A list containing the process IDs that need to be killed.
+        """
+        #The below is repeatedly tried since KILL is unreliable and we
+        #need to loop until all the processes are eventually terminated.
+        #An intersection with the set of processes currently running in
+        #the system, gives the processes on which the KILL has still not
+        #worked. Hence we need to keep calling KILL repeatedly on these.
+        #for e.g.
+        #P = {1, 2, 3, 4, 5, 6}
+        #p = {1, 2, 3}
+        #p &= P => {1,2,3}
+        #After the first KILL
+        #P = {3, 4, 5, 6}
+        #p = {1, 2, 3}
+        #p &= P => {3}
+        #Now we run KILL on {3}
+        #Thus each time we work with the set of proecesses in the original
+        #list that are still running.
+        proc_ids_set = set(proc_ids)
+        proc_ids_set.intersection_update(set(self.processes(True, True)))
+        while proc_ids_set:
+            for id in proc_ids_set:
+                self.exec_stmt("KILL %s", {"params": (id,)})
+            #sleep to ensure that the kill command reflects its results
+            time.sleep(math.log10(len(proc_ids_set)))
+            proc_ids_set.intersection_update(
+                set(self.processes(True, True))
+            )
+
     def exec_stmt(self, stmt_str, options=None):
         """Execute statements against the server.
         See :meth:`~mysql.fabric.server_utils.exec_stmt`.
@@ -1343,27 +1434,36 @@ class MySQLServer(_persistence.Persistable):
             )
 
     @staticmethod
-    def fetch(uuid, persister=None):
+    def fetch(server_id, persister=None):
         """Return a server object corresponding to the uuid.
 
-        :param uuid: The server id of the server object that needs to be
-                     returned.
+        :param server_id: Id or address of the server object that needs to be
+                          returned.
         :param persister: Persister to persist the object to.
         :return: The server object that corresponds to the server id
                  None if the server id does not exist.
         """
-        cur = persister.exec_stmt(MySQLServer.QUERY_SERVER,
-            {"fetch" : False, "raw" : False, "params":(str(uuid),)}
-        )
+        if server_id is None:
+            return
+
+        query = None
+        try:
+            if isinstance(server_id, _uuid.UUID):
+                server_id = str(server_id)
+            else:
+                _uuid.UUID(server_id)
+            query = MySQLServer.QUERY_SERVER_BY_UUID
+        except (ValueError, TypeError, AttributeError):
+            query = MySQLServer.QUERY_SERVER_BY_ADDRESS
+
+        cur = persister.exec_stmt(query, {
+            "fetch" : False,
+            "params" : (server_id, )
+        })
+
         row = cur.fetchone()
         if row:
-            uuid, address, idx_mode, idx_status, weight, group_id = row
-            mode = MySQLServer.get_mode(idx_mode)
-            status = MySQLServer.get_status(idx_status)
-            return MySQLServer(
-                _uuid.UUID(uuid), address=address, mode=mode, status=status,
-                weight=weight, group_id=group_id
-            )
+            return MySQLServer(row=row)
 
     @staticmethod
     def dump_servers(version=None, patterns="", persister=None):
@@ -1376,9 +1476,6 @@ class MySQLServer(_persistence.Persistable):
         :return: A list of servers belonging to the groups that match
                 the provided pattern.
         """
-        #This is used to store the consolidated list of servers
-        #that will be returned.
-        result_server_list = []
 
         #This stores the pattern that will be passed to the LIKE MySQL
         #command.
@@ -1398,18 +1495,15 @@ class MySQLServer(_persistence.Persistable):
             else:
                 like_pattern = '%' + find + '%'
             rows = persister.exec_stmt(MySQLServer.DUMP_SERVERS,
-                {"raw" : False, "params":(like_pattern, "FAULTY")})
+                {"params":(like_pattern, "FAULTY")}
+            )
 
             for row in rows:
                 host, port = _server_utils.split_host_port(
                                 row[2],
                                 _server_utils.MYSQL_DEFAULT_PORT
                              )
-                result_server_list.append(
-                    (row[0], row[1], host, port, row[3], row[4],
-                    row[5])
-                )
-        return _utils.wrap_output(result_server_list)
+                yield (row[0], row[1], host, port, row[3], row[4], row[5])
 
     @staticmethod
     def create(persister=None):
