@@ -69,6 +69,11 @@ import uuid as _uuid
 
 import mysql.fabric.services.utils as _utils
 
+from mysql.connector.errorcode import (
+    ER_ROW_IS_REFERENCED,
+    ER_ROW_IS_REFERENCED_2,
+)
+
 from mysql.fabric import (
     backup as _backup,
     events as _events,
@@ -78,11 +83,14 @@ from mysql.fabric import (
     group_replication as _group_replication,
     sharding as _sharding,
     server_utils as _server_utils,
+    config as _config,
 )
 
 from mysql.fabric.command import (
     ProcedureGroup,
     Command,
+    ResultSet,
+    CommandResult,
 )
 
 from mysql.fabric.utils import (
@@ -95,6 +103,10 @@ SERVER_NOT_FOUND = "Server with UUID %s not found."
 MYSQLDUMP_NOT_FOUND = "Unable to find MySQLDump in location %s"
 MYSQLCLIENT_NOT_FOUND = "Unable to find MySQL Client in location %s"
 
+MIN_UNREACHABLE_TIMEOUT = 1
+MAX_UNREACHABLE_TIMEOUT = 60
+DEFAULT_UNREACHABLE_TIMEOUT = 5
+
 class GroupLookups(Command):
     """Return information on existing group(s).
     """
@@ -109,9 +121,34 @@ class GroupLookups(Command):
         :return: List with {"group_id" : group_id, "failure_detector": ON/OFF,
                  "description" : description}.
         """
-        return Command.generate_output_pattern(
-            _lookup_groups, group_id)
 
+        if group_id is None:
+            gids = _server.Group.groups()
+        else:
+            gids = [group_id]
+
+        _LOGGER.debug("Group IDs: %s", gids)
+
+        # Fetch all the groups before building the result set since an
+        # exception can be thrown and there is little point in trying
+        # to build a result set before all groups can be fetched.
+        groups = [ _retrieve_group(gid) for gid in gids ]
+
+        rset = ResultSet(
+            names=('group_id', 'description', 'failure_detector', 'master_uuid'),
+            types=(str, str, bool, str)
+        )
+
+        for group in groups:
+            rset.append_row([
+                group.group_id,          # group_id
+                group.description,       # description
+                group.status,            # failure_detector
+                group.master,            # master_uuid
+            ])
+
+        return CommandResult(None, results=rset)
+            
 CREATE_GROUP = _events.Event()
 class GroupCreate(ProcedureGroup):
     """Create a group.
@@ -161,17 +198,16 @@ class DestroyGroup(ProcedureGroup):
     group_name = "group"
     command_name = "destroy"
 
-    def execute(self, group_id, force=False, synchronous=True):
+    def execute(self, group_id, synchronous=True):
         """Remove a group.
 
         :param group_id: Group's id.
-        :param force: If the group is not empty, remove it serves.
         :param synchronous: Whether one should wait until the execution finishes
                             or not.
         :return: Tuple with job's uuid and status.
         """
         procedures = _events.trigger(
-            DESTROY_GROUP, self.get_lockable_objects(), group_id, force
+            DESTROY_GROUP, self.get_lockable_objects(), group_id
         )
         return self.wait_for_procedures(procedures, synchronous)
 
@@ -194,9 +230,41 @@ class ServerLookups(Command):
         :return: Information on servers.
         :rtype: List with [uuid, address, status, mode, weight]
         """
-        return Command.generate_output_pattern(
-            _lookup_servers, group_id, server_id, status, mode
+        # Determine the set of servers to iterate through.
+        group = _retrieve_group(group_id)
+        if server_id is None:
+            servers = [server for server in group.servers()]
+        else:
+            servers = [_retrieve_server(server_id, group_id)]
+
+        # Determine the set of status to check upon.
+        if status is None:
+            status = _server.MySQLServer.SERVER_STATUS
+        else:
+            status = [_retrieve_server_status(status)]
+
+        # Determine the set of modes to check upon.
+        if mode is None:
+            mode = _server.MySQLServer.SERVER_MODE
+        else:
+            mode = [_retrieve_server_mode(mode)]
+
+        # Create result set.
+        rset = ResultSet(
+            names=('server_uuid', 'address', 'status', 'mode', 'weight'),
+            types=(str, str, str, str, float),
         )
+        for server in servers:
+            if server.status in status and server.mode in mode:
+                rset.append_row([
+                    str(server.uuid),
+                    server.address,
+                    server.status,
+                    server.mode,
+                    server.weight
+                ])
+
+        return CommandResult(None, results=rset)
 
 class ServerUuid(Command):
     """Return server's uuid.
@@ -204,7 +272,7 @@ class ServerUuid(Command):
     group_name = "server"
     command_name = "lookup_uuid"
 
-    def execute(self, address, timeout=5):
+    def execute(self, address, timeout=None):
         """Return server's UUID.
 
         :param address: Server's address.
@@ -212,7 +280,9 @@ class ServerUuid(Command):
                         if the UUID is not retrieved.
         :return: UUID.
         """
-        return Command.generate_output_pattern(_lookup_uuid, address, timeout)
+        rset = ResultSet(names=['uuid'], types=[str])
+        rset.append_row([_lookup_uuid(address, timeout)])
+        return CommandResult(None, results=rset)
 
 ADD_SERVER = _events.Event()
 class ServerAdd(ProcedureGroup):
@@ -228,7 +298,7 @@ class ServerAdd(ProcedureGroup):
     group_name = "group"
     command_name = "add"
 
-    def execute(self, group_id, address, timeout=5, update_only=False,
+    def execute(self, group_id, address, timeout=None, update_only=False,
                 synchronous=True):
         """Add a server into a group.
 
@@ -326,7 +396,16 @@ class DumpServers(Command):
         :param connector_version: The connectors version of the data.
         :param patterns: group pattern.
         """
-        return _server.MySQLServer.dump_servers(connector_version, patterns)
+        rset = ResultSet(
+            names=('server_uuid', 'group_id', 'host', 'port', 'mode', 'status', 'weight'),
+            types=(str, str, str, int, int, int, float)
+        )
+        
+        for row in _server.MySQLServer.dump_servers(connector_version, patterns):
+            rset.append_row(row)
+
+        return CommandResult(None, results=rset)
+
 
 SET_SERVER_STATUS = _events.Event()
 class SetServerStatus(ProcedureGroup):
@@ -420,31 +499,27 @@ class CloneServer(ProcedureGroup):
     group_name = "server"
     command_name = "clone"
 
-    def execute(self, group_id, destn_address, server_uuid=None,
+    def execute(self, group_id, destn_address, server_id=None, timeout=None,
                 synchronous=True):
         """Clone the objects of a given server into a destination server.
 
         :param group_id: The ID of the source group.
-        :param destn_address: The address of the MySQL Server to which
-            the clone needs to happen.
-        :param server_uuid: The UUID of the source MySQL Server.
+        :param destn_address: The address of the destination MySQL Server.
+        :param source_id: The address or UUID of the source MySQL Server.
+        :param timeout: Time in seconds after which an error is reported
+                        if the destination server is unreachable.
         :param synchronous: Whether one should wait until the execution
                             finishes or not.
         """
-        #If the destination server is already part of a Fabric Group, raise
-        #an error
-        if destn_address:
-            destn_server_uuid = _server.MySQLServer.\
-                discover_uuid(destn_address)
-            destn_server = _server.MySQLServer.fetch(destn_server_uuid)
-            #we should check for both the presence of the server object
-            #and its associated group ID. Checking its association with
-            #a group ID would verify that a server that is part of Fabric
-            #but is not part of a group can also be cloned into.
-            if destn_server and destn_server.group_id:
-                raise _errors.ServerError("The Destination server is already "\
-                    "part of Group (%s)" % (destn_server.group_id,))
+        # If the destination server is already part of a Fabric Group, raise
+        # an error
+        destn_server_uuid = _lookup_uuid(destn_address, timeout)
+        _check_server_not_in_any_group(destn_server_uuid)
+        host, port = _server_utils.split_host_port(
+            destn_address, _backup.MySQLDump.MYSQL_DEFAULT_PORT
+        )
 
+        # Fetch information on backup and restore programs.
         config_file = self.config.config_file if self.config.config_file else ""
 
         mysqldump_binary = _utils.read_config_value(
@@ -464,25 +539,18 @@ class CloneServer(ProcedureGroup):
         if not _utils.is_valid_binary(mysqlclient_binary):
             raise _errors.ServerError(MYSQLCLIENT_NOT_FOUND % mysqlclient_binary)
 
-        if server_uuid:
-            server = _server.MySQLServer.fetch(server_uuid)
-            if group_id != server.group_id:
-                raise _errors.ServerError("The server %s was not found in "\
-                                          "group %s" % (server_uuid, group_id, ))
-        elif not server_uuid:
-            group = _server.Group.fetch(group_id)
+        # Fetch a reference to source server.
+        if server_id:
+            server = _retrieve_server(server_id, group_id)
+        else:
+            group = _retrieve_group(group_id)
             server = _utils.fetch_backup_server(group)
-            server_uuid = str(server.uuid)
-
-        host, port = _server_utils.split_host_port(
-                                destn_address,
-                                _backup.MySQLDump.MYSQL_DEFAULT_PORT
-                            )
 
+        # Schedule the clone operation through the executor.
         procedures = _events.trigger(
             BACKUP_SERVER,
             self.get_lockable_objects(),
-            server_uuid,
+            str(server.uuid),
             host,
             port,
             mysqldump_binary,
@@ -491,29 +559,6 @@ class CloneServer(ProcedureGroup):
         )
         return self.wait_for_procedures(procedures, synchronous)
 
-def _lookup_groups(group_id=None):
-    """Return a list of existing group(s).
-    """
-    info = []
-    if group_id is None:
-        for group_id in _server.Group.groups():
-            _group_information(_server.Group.fetch(group_id[0]), info)
-    else:
-        group = _retrieve_group(group_id)
-        _group_information(group, info)
-    return info
-
-def _group_information(group, info):
-    """Get information on group and append it into to a list.
-    """
-    master = str(group.master) if group.master else ""
-    info.append({
-        "group_id" : group.group_id,
-        "description" : group.description or "",
-        "failure_detector" : True if group.status else False,
-        "master_uuid" : master
-    })
-
 @_events.on_event(CREATE_GROUP)
 def _create_group(group_id, description):
     """Create a group.
@@ -553,72 +598,41 @@ def _update_group_description(group_id, description):
     _LOGGER.debug("Updated group (%s).", group)
 
 @_events.on_event(DESTROY_GROUP)
-def _destroy_group(group_id, force):
+def _destroy_group(group_id):
     """Destroy a group.
     """
     group = _retrieve_group(group_id)
-
-    _check_shard_exists(group_id)
-
-    servers = group.servers()
-    if servers and force:
-        for server in servers:
-            _server.MySQLServer.remove(server)
-    elif servers:
-        raise _errors.GroupError("Group (%s) is not empty." % (group_id, ))
-
+    _check_group_dependencies(group)
     _detector.FailureDetector.unregister_group(group_id)
-    _server.Group.remove(group)
-    _LOGGER.debug("Removed group (%s).", group)
+    try:
+        _server.Group.remove(group)
+    except _errors.DatabaseError as error:
+        foreign_key_errors = (ER_ROW_IS_REFERENCED, ER_ROW_IS_REFERENCED_2)
+        if error.errno in foreign_key_errors:
+            raise _errors.GroupError(
+                "Cannot destroy group (%s): %s." % (group_id, error, )
+            )
+        raise
+    _LOGGER.debug("Destroyed group (%s).", group)
 
-def _lookup_servers(group_id, server_id=None, status=None, mode=None):
-    """Return existing servers in a group or information on a server.
+@_destroy_group.undo
+def _undo_destroy_group(group_id):
+    """Register a group if the destroy operation has failed.
     """
-    group = _retrieve_group(group_id)
-
-    status = _retrieve_server_status(status) if status is not None else None
-    if status is None:
-        status = _server.MySQLServer.SERVER_STATUS
-    else:
-        status = [status]
-
-    mode = _retrieve_server_mode(mode) if mode is not None else None
-    if mode is None:
-        mode = _server.MySQLServer.SERVER_MODE
-    else:
-        mode = [mode]
-
-    info = []
-    if server_id is None:
-        for server in group.servers():
-            if server.status in status and server.mode in mode:
-                _server_information(server, info)
-    else:
-        server = _retrieve_server(server_id, group_id)
-        _server_information(server, info)
-
-    return info
-
-def _server_information(server, info):
-    """Get information on server and append it into to a list.
-    """
-    info.append({
-        "server_uuid" : str(server.uuid),
-        "address" : server.address,
-        "status" : server.status,
-        "mode" : server.mode,
-        "weight" : server.weight,
-    })
+    _detector.FailureDetector.register_group(group_id)
 
 def _lookup_uuid(address, timeout):
     """Return server's uuid.
     """
+    timeout = timeout or DEFAULT_UNREACHABLE_TIMEOUT
     try:
         return _server.MySQLServer.discover_uuid(
             address=address, connection_timeout=timeout
         )
-    except _errors.DatabaseError:
-        raise _errors.ServerError("Error accessing server (%s)." % (address, ))
+    except _errors.DatabaseError as error:
+        raise _errors.ServerError(
+            "Error accessing server (%s): %s." % (address, error)
+        )
 
 @_events.on_event(ADD_SERVER)
 def _add_server(group_id, address, timeout, update_only):
@@ -934,25 +948,25 @@ def _do_set_server_mode(server, mode, allowed_mode):
     server.mode = mode
 
 def _retrieve_server(server_id, group_id=None):
-    """Return a MySQLServer object from a UUID or an HOST:PORT.
+    """Return a MySQLServer object from a UUID or a HOST:PORT.
     """
-    uuid = _retrieve_uuid_object(server_id)
+    server = _server.MySQLServer.fetch(server_id)
 
-    server = _server.MySQLServer.fetch(uuid)
     if not server:
         raise _errors.ServerError(
-            "Server (%s) does not exist." % (uuid, )
+            "Server (%s) does not exist." % (server_id, )
             )
 
     if not server.group_id:
         raise _errors.GroupError(
-            "Server (%s) does not belong to a group." % (uuid, )
+            "Server (%s) does not belong to a group." % (server_id, )
             )
 
     if group_id is not None and group_id != server.group_id:
         raise _errors.GroupError(
-            "Group (%s) does not contain server (%s)." % (group_id, uuid)
+            "Group (%s) does not contain server (%s)." % (group_id, server_id)
         )
+
     return server
 
 def _check_server_exists(server_id):
@@ -960,12 +974,24 @@ def _check_server_exists(server_id):
 
     :param server_id: The UUID or the host:port for the server.
     """
-    server_id = _retrieve_uuid_object(server_id)
-
     server = _server.MySQLServer.fetch(server_id)
+
     if server:
         raise _errors.ServerError("Server (%s) already exists." % (server_id, ))
 
+def _check_server_not_in_any_group(server_id):
+    """Check for both the presence of the server object and its associated
+       group. If the server belongs to a group an error is raised.
+
+    :param server_id: The UUID or the host:port for the server.
+    """
+    server = _server.MySQLServer.fetch(server_id)
+    if server and server.group_id:
+        raise _errors.ServerError(
+            "Destination server (%s) is already part of group (%s)" %
+            (server.address, server.group_id)
+        )
+
 def _retrieve_group(group_id):
     """Return a Group object from an identifier.
     """
@@ -981,38 +1007,29 @@ def _check_group_exists(group_id):
     if group:
         raise _errors.GroupError("Group (%s) already exists." % (group_id, ))
 
-def _check_shard_exists(group_id):
+def _check_group_dependencies(group):
     """Check whether there is a shard associated with the group.
     """
+    group_id = group.group_id
+
     shard_id = _sharding.Shards.lookup_shard_id(group_id)
     if shard_id:
         raise _errors.GroupError(
-            "Cannot erase a group (%s) which is associated to a shard (%s)." % 
+            "Cannot destroy a group (%s) which is associated to a shard (%s)." %
             (group_id, shard_id)
         )
 
     shard_mapping_id = _sharding.ShardMapping.lookup_shard_mapping_id(group_id)
     if shard_mapping_id:
         raise _errors.GroupError(
-            "Cannot erase a group (%s) which is used as a global group in a "
+            "Cannot destroy a group (%s) which is used as a global group in a "
             "shard definition (%s)." % (group_id, shard_mapping_id)
         )
 
-def _retrieve_uuid_object(server_id):
-    """Transform an input string into a UUID object.
-    """
-    assert(isinstance(server_id, basestring))
-
-    try:
-        return _uuid.UUID(server_id)
-    except ValueError:
-        pass
-
-    try:
-        return _server.MySQLServer.discover_uuid(address=server_id)
-    except ValueError:
-        raise _errors.ServerError(
-            "Error trying to access server identified by (%s)." % (server_id, )
+    if group.servers():
+        raise _errors.GroupError(
+            "Cannot destroy a group (%s) which has associated servers." %
+            (group_id, )
         )
 
 def _check_requirements(server):
@@ -1025,7 +1042,7 @@ def _check_requirements(server):
         raise _errors.ServerError(
             "Server (%s) has an outdated version (%s). 5.6.8 or greater "
             "is required." % (server.uuid, server.version)
-            )
+        )
 
     if not server.has_required_privileges():
         raise _errors.ServerError(
@@ -1039,7 +1056,7 @@ def _check_requirements(server):
         raise _errors.ServerError(
             "Server (%s) does not have the binary log or gtid enabled."
             % (server.uuid, )
-            )
+        )
 
 def _configure_as_slave(group, server):
     """Configure the server as a slave.
@@ -1050,10 +1067,29 @@ def _configure_as_slave(group, server):
             master.connect()
             _utils.switch_master(server, master)
     except _errors.DatabaseError as error:
-        _LOGGER.debug(
-            "Error configuring slave (%s)...", server.uuid, exc_info=error
-        )
-        raise _errors.ServerError(
-            "Error trying to configure Server (%s) as slave."
-            % (server.uuid, )
-        )
+        msg = "Error trying to configure server ({0}) as slave: {1}.".format(
+            server.uuid, error)
+        _LOGGER.debug(msg)
+        raise _errors.ServerError(msg)
+
+def configure(config):
+    """Set configuration values.
+    """
+    try:
+        timeout = int(config.get("servers", "unreachable_timeout"))
+        if timeout < MIN_UNREACHABLE_TIMEOUT:
+            _LOGGER.warning(
+                "Unreachable timeout cannot be lower than %s.",
+                MIN_UNREACHABLE_TIMEOUT
+            )
+            timeout = MIN_UNREACHABLE_TIMEOUT
+        if timeout > MAX_UNREACHABLE_TIMEOUT:
+            _LOGGER.warning(
+                "Unreachable timeout cannot be greater than %s.",
+                MAX_UNREACHABLE_TIMEOUT
+            )
+            timeout = MAX_UNREACHABLE_TIMEOUT
+        global DEFAULT_UNREACHABLE_TIMEOUT
+        DEFAULT_UNREACHABLE_TIMEOUT = timeout
+    except (_config.NoOptionError, _config.NoSectionError, ValueError):
+        pass
