@@ -192,6 +192,11 @@ class RplCommands(object):
         self.candidates = self.options.get("candidates", None)
         self.verbose = self.options.get("verbose", None)
         self.rpl_user = self.options.get("rpl_user", None)
+        self.ssl_ca = options.get("ssl_ca", None)
+        self.ssl_cert = options.get("ssl_cert", None)
+        self.ssl_key = options.get("ssl_key", None)
+        if self.ssl_ca or self.ssl_cert or self.ssl_key:
+            self.ssl = True
 
         try:
             self.topology = Topology(master_vals, slave_vals, self.options,
@@ -356,6 +361,31 @@ class RplCommands(object):
             self._report(msg, logging.CRITICAL)
             raise UtilRplError(msg)
 
+        # Can only check errant transactions if GTIDs are enabled.
+        if self.topology.gtid_enabled():
+            # Check existence of errant transactions on slaves
+            errant_tnx = self.topology.find_errant_transactions()
+            if errant_tnx:
+                force = self.options.get('force')
+                print("# ERROR: {0}".format(_ERRANT_TNX_ERROR))
+                self._report(_ERRANT_TNX_ERROR, logging.ERROR, False)
+                for host, port, tnx_set in errant_tnx:
+                    errant_msg = (" - For slave '{0}@{1}': "
+                                  "{2}".format(host, port, ", ".join(tnx_set)))
+                    print("# {0}".format(errant_msg))
+                    self._report(errant_msg, logging.ERROR, False)
+                # Raise an exception (to stop) if tolerant mode is OFF
+                if not force:
+                    raise UtilRplError("{0} Note: If you want to ignore this "
+                                       "issue, although not advised, please "
+                                       "use the utility with the --force "
+                                       "option.".format(_ERRANT_TNX_ERROR))
+        else:
+            warn_msg = ("Errant transactions check skipped (GTID not enabled "
+                        "for the whole topology).")
+            print("# WARNING: {0}".format(warn_msg))
+            self._report(warn_msg, logging.WARN, False)
+
         self._report(" ".join(["# Performing switchover from master at",
                      "%s:%s" % (self.master_vals['host'],
                                 self.master_vals['port']),
@@ -443,10 +473,10 @@ class RplCommands(object):
                 self._report(errant_msg, logging.ERROR, False)
             # Raise an exception (to stop) if tolerant mode is OFF
             if not force:
-                raise UtilRplError("%s Note: If you want to ignore this issue,"
-                                   " although not advised, please use the "
-                                   "utility with the --force option."
-                                   % _ERRANT_TNX_ERROR)
+                raise UtilRplError("{0} Note: If you want to ignore this "
+                                   "issue, although not advised, please use "
+                                   "the utility with the --force option."
+                                   "".format(_ERRANT_TNX_ERROR))
 
         self._report("# Performing failover.")
         if not self.topology.failover(self.candidates, strict,
