@@ -30,6 +30,7 @@ import tempfile
 import threading
 
 import mysql.connector
+from mysql.connector.constants import ClientFlag
 
 from mysql.connector.errorcode import CR_SERVER_LOST
 from mysql.utilities.exception import (ConnectionValuesError, UtilError,
@@ -47,7 +48,59 @@ _GTID_ERROR = ("The server %s:%s does not comply to the latest GTID "
                "feature support. Errors:")
 
 
-def get_connection_dictionary(conn_info):
+def tostr(value):
+    """Cast value to str except when None
+
+    value[in]          Value to be cast to str
+
+    Returns value as str instance or None.
+    """
+    return None if value is None else str(value)
+
+
+class MySQLUtilsCursorRaw(mysql.connector.cursor.MySQLCursorRaw):
+    """
+    Cursor for Connector/Python v2.0, returning str instead of bytearray
+    """
+    def fetchone(self):
+        row = self._fetch_row()
+        if row:
+            return tuple([tostr(v) for v in row])
+        return None
+
+    def fetchall(self):
+        rows = []
+        all_rows = super(MySQLUtilsCursorRaw, self).fetchall()
+        for row in all_rows:
+            rows.append(tuple([tostr(v) for v in row]))
+        return rows
+
+
+class MySQLUtilsCursorBufferedRaw(
+        mysql.connector.cursor.MySQLCursorBufferedRaw):
+    """
+    Cursor for Connector/Python v2.0, returning str instead of bytearray
+    """
+    def fetchone(self):
+        row = self._fetch_row()
+        if row:
+            return tuple([tostr(v) for v in row])
+        return None
+
+    def fetchall(self):
+        if self._rows is None:
+            raise mysql.connector.InterfaceError(
+                "No result set to fetch from."
+            )
+
+        rows = []
+        all_rows = [r for r in self._rows[self._next_row:]]
+        for row in all_rows:
+            rows.append(tuple([tostr(v) for v in row]))
+        return rows
+
+
+def get_connection_dictionary(conn_info, ssl_dict=None):
     """Get the connection dictionary.
 
     The method accepts one of the following types for conn_info:
@@ -59,20 +112,29 @@ def get_connection_dictionary(conn_info):
         - an instance of the Server class
 
     conn_info[in]          Connection information
+    ssl_dict[in]           A dictionary with the ssl certificates
+                           (ssl_ca, ssl_cert and ssl_key).
 
     Returns dict - dictionary for connection (user, passwd, host, port, socket)
     """
     if conn_info is None:
         return conn_info
+
     conn_val = {}
     if isinstance(conn_info, dict) and 'host' in conn_info:
+        # Not update conn_info if already has any ssl certificate.
+        if (ssl_dict is not None and
+            not (conn_info.get("ssl_ca", None) or
+                 conn_info.get("ssl_cert", None) or
+                 conn_info.get("ssl_key", None))):
+            conn_info.update(ssl_dict)
         conn_val = conn_info
     elif isinstance(conn_info, Server):
         # get server's dictionary
         conn_val = conn_info.get_connection_values()
     elif isinstance(conn_info, basestring):
         # parse the string
-        conn_val = parse_connection(conn_info, None)
+        conn_val = parse_connection(conn_info, options=ssl_dict)
     else:
         raise ConnectionValuesError("Cannot determine connection information"
                                     " type.")
@@ -185,15 +247,17 @@ def get_local_servers(all_proc=False, start=3306, end=3333,
     return processes
 
 
-def get_server(name, values, quiet):
+def get_server(name, values, quiet, verbose=False):
     """Connect to a server and return Server instance
 
     If the name is 'master' or 'slave', the connection will be made via the
     Master or Slave class else a normal Server class shall be used.
 
-    name[in]           name of the server
-    values[in]         dictionary of connection values
-    quiet[in]          if True, do not print messages
+    name[in]        Name of the server.
+    values[in]      Dictionary of connection values.
+    quiet[in]       If True, do not print messages.
+    verbose[in]     Verbose value used by the returned server instances.
+                    By default False.
 
     Returns Server class instance
     """
@@ -208,6 +272,7 @@ def get_server(name, values, quiet):
     server_options = {
         'conn_info': values,
         'role': name,
+        'verbose': verbose,
     }
     if name.lower() == 'master':
         server_conn = Master(server_options)
@@ -215,7 +280,12 @@ def get_server(name, values, quiet):
         server_conn = Slave(server_options)
     else:
         server_conn = Server(server_options)
-    server_conn.connect()
+    try:
+        server_conn.connect()
+    except:
+        if not quiet:
+            print("")
+        raise
 
     return server_conn
 
@@ -282,7 +352,8 @@ def connect_servers(src_val, dest_val, options=None):
         - dictionary containing connection information including:
           (user, passwd, host, port, socket)
         - connection string in the form: user:pass@host:port:socket or
-                                         login-path:port:socket
+                                         login-path:port:socket or
+                                         config-path[group]
         - an instance of the Server class
 
     src_val[in]        source connection information
@@ -299,6 +370,8 @@ def connect_servers(src_val, dest_val, options=None):
                        (default is "Destination")
         unique         if True, servers must be different when dest_val is
                        not None (default is False)
+        verbose        Verbose value used by the returned server instances
+                       (default is False).
 
     Returns tuple (source, destination) where
             source = connection to source server
@@ -313,12 +386,21 @@ def connect_servers(src_val, dest_val, options=None):
     dest_name = options.get("dest_name", "Destination")
     version = options.get("version", None)
     charset = options.get("charset", None)
+    verbose = options.get('verbose', False)
+
+    ssl_dict = {}
+    if options.get("ssl_cert", None) is not None:
+        ssl_dict['ssl_cert'] = options.get("ssl_cert")
+    if options.get("ssl_ca", None) is not None:
+        ssl_dict['ssl_ca'] = options.get("ssl_ca", None)
+    if options.get("ssl_key", None) is not None:
+        ssl_dict['ssl_key'] = options.get("ssl_key", None)
 
     source = None
     destination = None
 
     # Get connection dictionaries
-    src_dict = get_connection_dictionary(src_val)
+    src_dict = get_connection_dictionary(src_val, ssl_dict)
     if "]" in src_dict['host']:
         src_dict['host'] = clean_IPv6(src_dict['host'])
     dest_dict = get_connection_dictionary(dest_val)
@@ -350,7 +432,7 @@ def connect_servers(src_val, dest_val, options=None):
     if isinstance(src_val, Server):
         source = src_val
     else:
-        source = get_server(src_name, src_dict, quiet)
+        source = get_server(src_name, src_dict, quiet, verbose=verbose)
         if not quiet:
             print "connected."
     if not _require_version(source, version):
@@ -363,7 +445,8 @@ def connect_servers(src_val, dest_val, options=None):
         if isinstance(dest_val, Server):
             destination = dest_val
         else:
-            destination = get_server(dest_name, dest_dict, quiet)
+            destination = get_server(dest_name, dest_dict, quiet,
+                                     verbose=verbose)
             if not quiet:
                 print "connected."
         if not _require_version(destination, version):
@@ -372,12 +455,16 @@ def connect_servers(src_val, dest_val, options=None):
                             (dest_name, version))
     elif not quiet and dest_dict is not None and \
             not isinstance(dest_val, Server):
-        _print_connection(dest_name, src_dict)
-        print "connected."
+        try:
+            _print_connection(dest_name, src_dict)
+            print "connected."
+        except:
+            print("")
+            raise
     return (source, destination)
 
 
-def test_connect(conn_info, throw_errors=False):
+def test_connect(conn_info, throw_errors=False, ssl_dict=None):
     """Test connection to a server.
 
     The method accepts one of the following types for conn_info:
@@ -385,19 +472,22 @@ def test_connect(conn_info, throw_errors=False):
         - dictionary containing connection information including:
           (user, passwd, host, port, socket)
         - connection string in the form: user:pass@host:port:socket or
-                                         login-path:port:socket
+                                         login-path:port:socket or
+                                         config-path[group]
         - an instance of the Server class
 
     conn_info[in]          Connection information
 
     throw_errors           throw any errors found during the test,
                            false by default.
+    ssl_dict[in]           A dictionary with the ssl certificates
+                           (ssl_ca, ssl_cert and ssl_key).
 
     Returns True if connection success, False if error
     """
     # Parse source connection values
     try:
-        src_val = get_connection_dictionary(conn_info)
+        src_val = get_connection_dictionary(conn_info, ssl_dict)
     except Exception as err:
         raise ConnectionValuesError("Server connection values invalid: {0}."
                                     "".format(err))
@@ -470,14 +560,22 @@ def stop_running_server(server, wait=10, drop=True):
     cmd += mysqladmin_path
     if server.socket is None and server.host == 'localhost':
         server.host = '127.0.0.1'
-    cmd += " shutdown --user=%s --host=%s " % (server.user, server.host)
+    cmd += " shutdown --user={0} --host={1} ".format(server.user, server.host)
     if server.passwd:
-        cmd += "--password=%s " % server.passwd
+        cmd = "{0} --password={1} ".format(cmd, server.passwd)
     # Use of server socket only works with 'localhost' (not with 127.0.0.1).
     if server.socket and server.host == 'localhost':
-        cmd += "--socket=%s " % server.socket
+        cmd = "{0} --socket={1} ".format(cmd, server.socket)
     else:
-        cmd += "--port=%s " % server.port
+        cmd = "{0} --port={1} ".format(cmd, server.port)
+    if server.has_ssl:
+        if server.ssl_cert is not None:
+            cmd = "{0} --ssl-cert={1} ".format(cmd, server.ssl_cert)
+        if server.ssl_ca is not None:
+            cmd = "{0} --ssl-ca={1} ".format(cmd, server.ssl_ca)
+        if server.ssl_key is not None:
+            cmd = "{0} --ssl-key={1} ".format(cmd, server.ssl_key)
+
     res = server.show_server_variable("datadir")
     datadir = res[0][1]
 
@@ -561,6 +659,7 @@ class Server(object):
         self.db_conn = None
         self.host = None
         self.role = options.get("role", "Server")
+        self.has_ssl = False
         conn_values = get_connection_dictionary(options.get("conn_info"))
         try:
             self.host = conn_values["host"]
@@ -574,6 +673,12 @@ class Server(object):
                 self.port = int(conn_values["port"])
             self.charset = options.get("charset",
                                        conn_values.get("charset", None))
+            # Optional values
+            self.ssl_ca = conn_values.get('ssl_ca', None)
+            self.ssl_cert = conn_values.get('ssl_cert', None)
+            self.ssl_key = conn_values.get('ssl_key', None)
+            if self.ssl_cert or self.ssl_ca or self.ssl_key:
+                self.has_ssl = True
         except KeyError:
             raise UtilError("Dictionary format not recognized.")
         self.connect_error = None
@@ -582,28 +687,34 @@ class Server(object):
         self.fkeys = None
         self.autocommit = None
         self.read_only = False
-        self.aliases = []
+        self.aliases = set()
         self.grants_enabled = None
         self._version = None
 
     @classmethod
-    def fromServer(cls, server):
+    def fromServer(cls, server, conn_info=None):
         """ Create a new server instance from an existing one
 
         Factory method that will allow the creation of a new server instance
         from an existing server.
 
-        server[in] - instance object that must be instance of the Server class
-                     or a subclass.
+        server[in]       instance object that must be instance of the Server
+                         class or a subclass.
+        conn_info[in]    A dictionary with the connection information to
+                         connect to the server
 
         Returns an instance of the calling class as a result.
         """
 
         if isinstance(server, Server):
-            options = {"conn_info": server.get_connection_values(),
-                       "role": server.role,
+            options = {"role": server.role,
                        "verbose": server.verbose,
                        "charset": server.charset}
+            if conn_info is not None and isinstance(conn_info, dict):
+                options["conn_info"] = conn_info
+            else:
+                options["conn_info"] = server.get_connection_values()
+
             return cls(options)
         else:
             raise TypeError("The server argument's type is neither Server nor "
@@ -630,6 +741,115 @@ class Server(object):
             res = False
         return res
 
+    def _update_alias(self, ip_or_hostname, suffix_list):
+        """Update list of aliases for the given IP or hostname.
+
+        Gets the list of aliases for host *ip_or_hostname*. If any
+        of them matches one of the server's aliases, then update
+        the list of aliases (self.aliases). It also receives a list (tuple)
+        of suffixes that can be ignored when checking if two hostnames are
+        the same.
+
+        ip_or_hostname[in] IP or hostname to test.
+        suffix_list[in]    Tuple with list of suffixes that can be ignored.
+
+        Returns True if ip_or_hostname is a server alias, otherwise False.
+        """
+        host_or_ip_aliases = self._get_aliases(ip_or_hostname)
+        host_or_ip_aliases.add(ip_or_hostname)
+
+        # Check if any of aliases matches with one the servers's aliases
+        common_alias = self.aliases.intersection(host_or_ip_aliases)
+        if common_alias:  # There are common aliases, host is the same
+            self.aliases.update(host_or_ip_aliases)
+            return True
+        else:  # Check with and without suffixes
+            no_suffix_server_aliases = set()
+            no_suffix_host_aliases = set()
+
+            for suffix in suffix_list:
+                # Add alias with and without suffix from self.aliases
+                for alias in self.aliases:
+                    if alias.endswith(suffix):
+                        host, _ = alias.rsplit('.')
+                        no_suffix_server_aliases.add(host)
+                    no_suffix_server_aliases.add(alias)
+                # Add alias with and without suffix from host_aliases
+                for alias in host_or_ip_aliases:
+                    if alias.endswith(suffix):
+                        host, _ = alias.rsplit('.')
+                        no_suffix_host_aliases.add(host)
+                    no_suffix_host_aliases.add(alias)
+            # Check if there is any alias in common
+            common_alias = no_suffix_host_aliases.intersection(
+                no_suffix_server_aliases)
+            if common_alias:  # Same host, so update self.aliases
+                self.aliases.update(
+                    no_suffix_host_aliases.union(no_suffix_server_aliases)
+                )
+                return True
+
+        return False
+
+    def _get_aliases(self, host):
+        """Gets the aliases for the given host
+        """
+        aliases = set([clean_IPv6(host)])
+        if hostname_is_ip(clean_IPv6(host)):  # IP address
+            try:
+                my_host = socket.gethostbyaddr(clean_IPv6(host))
+                aliases.add(my_host[0])
+                # socket.gethostbyname_ex() does not work with ipv6
+                if (not my_host[0].count(":") < 1 or
+                   not my_host[0] == "ip6-localhost"):
+                    host_ip = socket.gethostbyname_ex(my_host[0])
+                else:
+                    addrinfo = socket.getaddrinfo(my_host[0], None)
+                    host_ip = ([socket.gethostbyaddr(addrinfo[0][4][0])],
+                               [fiveple[4][0] for fiveple in addrinfo],
+                               [addrinfo[0][4][0]])
+            except (socket.gaierror, socket.herror,
+                    socket.error) as err:
+                host_ip = ([], [], [])
+                if self.verbose:
+                    print("WARNING: IP lookup by address failed for {0},"
+                          "reason: {1}".format(host, err.strerror))
+        else:
+            try:
+                # server may not really exist.
+                host_ip = socket.gethostbyname_ex(host)
+            except (socket.gaierror, socket.herror,
+                    socket.error) as err:
+                if self.verbose:
+                    print("WARNING: hostname: {0} may not be reachable, "
+                          "reason: {1}".format(host, err.strerror))
+                return aliases
+            aliases.add(host_ip[0])
+            addrinfo = socket.getaddrinfo(host, None)
+            local_ip = None
+            error = None
+            for addr in addrinfo:
+                try:
+                    local_ip = socket.gethostbyaddr(addr[4][0])
+                    break
+                except (socket.gaierror, socket.herror,
+                        socket.error) as err:
+                    error = err
+
+            if local_ip:
+                host_ip = ([local_ip[0]],
+                           [fiveple[4][0] for fiveple in addrinfo],
+                           [addrinfo[0][4][0]])
+            else:
+                host_ip = ([], [], [])
+                if self.verbose:
+                    print("WARNING: IP lookup by name failed for {0},"
+                          "reason: {1}".format(host, error.strerror))
+        aliases.update(set(host_ip[1]))
+        aliases.update(set(host_ip[2]))
+
+        return aliases
+
     def is_alias(self, host_or_ip):
         """Determine if host_or_ip is an alias for this host
 
@@ -637,123 +857,49 @@ class Server(object):
 
         Returns bool - True = host_or_ip is an alias
         """
-        def get_aliases(host):
-            """Gets the aliases for the given host
-            """
-            aliases = [clean_IPv6(host)]
-            if hostname_is_ip(clean_IPv6(host)):  # IP address
-                try:
-                    my_host = socket.gethostbyaddr(clean_IPv6(host))
-                    aliases.append(my_host[0])
-                    # socket.gethostbyname_ex() does not work with ipv6
-                    if (not my_host[0].count(":") < 1 or
-                       not my_host[0] == "ip6-localhost"):
-                        host_ip = socket.gethostbyname_ex(my_host[0])
-                    else:
-                        addrinfo = socket.getaddrinfo(my_host[0], None)
-                        host_ip = ([socket.gethostbyaddr(addrinfo[0][4][0])],
-                                   [fiveple[4][0] for fiveple in addrinfo],
-                                   [addrinfo[0][4][0]])
-                except (socket.gaierror, socket.herror,
-                        socket.error) as err:
-                    host_ip = ([], [], [])
-                    if self.verbose:
-                        print("WARNING: IP lookup by address failed for {0},"
-                              "reason: {1}".format(host, err.strerror))
-            else:
-                try:
-                    # server may not really exist.
-                    host_ip = socket.gethostbyname_ex(host)
-                except (socket.gaierror, socket.herror,
-                        socket.error) as err:
-                    if self.verbose:
-                        print("WARNING: hostname: {0} may not be reachable, "
-                              "reason: {1}".format(host, err.strerror))
-                    return aliases
-                extend_aliases(aliases, [host_ip[0]])
-                addrinfo = socket.getaddrinfo(host, None)
-                local_ip = None
-                error = None
-                for addr in addrinfo:
-                    try:
-                        local_ip = socket.gethostbyaddr(addr[4][0])
-                        break
-                    except (socket.gaierror, socket.herror,
-                            socket.error) as err:
-                        error = err
-
-                if local_ip:
-                    host_ip = ([local_ip[0]],
-                               [fiveple[4][0] for fiveple in addrinfo],
-                               [addrinfo[0][4][0]])
-                else:
-                    host_ip = ([], [], [])
-                    if self.verbose:
-                        print("WARNING: IP lookup by name failed for {0},"
-                              "reason: {1}".format(host, error.strerror))
-            extend_aliases(aliases, host_ip[1])
-            extend_aliases(aliases, host_ip[2])
-
-            return aliases
-
-        def extend_aliases(aliases, als_new):
-            """Does intersection of the two given list.
-            Returns List of no repeated elements from the two list
-            """
-            if als_new:
-                als_new = set(als_new)  # Create a set to remove duplicates.
-                als_new = [alias for alias in als_new if (alias and
-                                                          alias not in aliases
-                                                          )]
-                aliases.extend(als_new)
+        # List of possible suffixes
+        suffixes = ('.local', '.lan', '.localdomain')
 
         host_or_ip = clean_IPv6(host_or_ip.lower())
 
-        # for quickness, verify in the existing  aliases, if they are.
+        # for quickness, verify in the existing  aliases, if they exist.
         if self.aliases:
             if host_or_ip.lower() in self.aliases:
                 return True
             else:
                 # get the alias for the given host_or_ip
-                host_or_ip_aliases = get_aliases(host_or_ip)
-                host_or_ip_aliases.append(host_or_ip)
-                for alias in host_or_ip_aliases:
-                    if alias in self.aliases:
-                        #save the aliases for future
-                        extend_aliases(self.aliases, host_or_ip_aliases)
-                        return True
-                return False
+                return self._update_alias(host_or_ip, suffixes)
 
-        # If not previews aliases save, look for them.
+        # no previous aliases information
         # First, get the local information
+        hostname_ = socket.gethostname()
         try:
-            local_info = socket.gethostbyname_ex(socket.gethostname())
-            local_aliases = [local_info[0].lower()]
+            local_info = socket.gethostbyname_ex(hostname_)
+            local_aliases = set([local_info[0].lower()])
             # if dotted host name, take first part and use as an alias
             try:
-                local_aliases.append(local_info[0].split('.')[0])
+                local_aliases.add(local_info[0].split('.')[0])
             except:
                 pass
-            extend_aliases(local_aliases, ['127.0.0.1', 'localhost',
-                                           '::1', '[::1]'])
-            extend_aliases(local_aliases, local_info[1])
-            extend_aliases(local_aliases, local_info[2])
-            extend_aliases(local_aliases, get_aliases(socket.gethostname()))
-        except (socket.herror, socket.gaierror, socket.error):
+            local_aliases.update(['127.0.0.1', 'localhost', '::1', '[::1]'])
+            local_aliases.update(local_info[1])
+            local_aliases.update(local_info[2])
+            local_aliases.update(self._get_aliases(hostname_))
+        except (socket.herror, socket.gaierror, socket.error) as err:
+            if self.verbose:
+                print("WARNING: Unable to find aliases for hostname"
+                      " '{0}' reason: {1}".format(hostname_, str(err)))
             # Try with the basic local aliases.
-            local_aliases = ['127.0.0.1', 'localhost', '::1', '[::1]']
+            local_aliases = set(['127.0.0.1', 'localhost', '::1', '[::1]'])
 
         # Get the aliases for this server host
-        self.aliases = get_aliases(self.host)
-
-        # List of possible suffixes
-        suffixes = ('.local', '.lan',)
+        self.aliases = self._get_aliases(self.host)
 
         # Check if this server is local
-        for host in self.aliases:
+        for host in self.aliases.copy():
             if host in local_aliases:
                 # Is local then save the local aliases for future.
-                extend_aliases(self.aliases, local_aliases)
+                self.aliases.update(local_aliases)
                 break
             # Handle special suffixes in hostnames.
             for suffix in suffixes:
@@ -763,22 +909,16 @@ class Server(object):
                     host, _ = host.rsplit('.', 1)
                     if host in local_aliases:
                         # Is local then save the local aliases for future.
-                        extend_aliases(self.aliases, local_aliases)
+                        self.aliases.update(local_aliases)
                         break
 
         # Check if the given host_or_ip is alias of the server host.
         if host_or_ip in self.aliases:
             return True
 
-        # lastly get the alias for the given host_or_ip
-        host_or_ip_aliases = get_aliases(host_or_ip)
-        extend_aliases(host_or_ip_aliases, [host_or_ip])
-        for alias in host_or_ip_aliases:
-            if alias in self.aliases:
-                #save the aliases for future
-                extend_aliases(self.aliases, host_or_ip_aliases)
-                return True
-        return False
+        # Check if any of the aliases of ip_or_host is also an alias of the
+        # host server.
+        return self._update_alias(host_or_ip, suffixes)
 
     def user_host_exists(self, user, host_or_ip):
         """Check to see if a user, host exists
@@ -814,6 +954,12 @@ class Server(object):
             conn_vals["socket"] = self.socket
         if self.port:
             conn_vals["port"] = self.port
+        if self.ssl_ca:
+            conn_vals["ssl_ca"] = self.ssl_ca
+        if self.ssl_cert:
+            conn_vals["ssl_cert"] = self.ssl_cert
+        if self.ssl_key:
+            conn_vals["ssl_key"] = self.ssl_key
 
         return conn_vals
 
@@ -867,12 +1013,25 @@ class Server(object):
                 parameters['charset'] = self.charset
             parameters['host'] = parameters['host'].replace("[", "")
             parameters['host'] = parameters['host'].replace("]", "")
+
+            # Add SSL parameters ONLY if they are not None
+            if self.ssl_ca is not None:
+                parameters['ssl_ca'] = self.ssl_ca
+            if self.ssl_cert is not None:
+                parameters['ssl_cert'] = self.ssl_cert
+            if self.ssl_key is not None:
+                parameters['ssl_key'] = self.ssl_key
+            if self.has_ssl:
+                cpy_flags = [ClientFlag.SSL, ClientFlag.SSL_VERIFY_SERVER_CERT]
+                parameters['client_flags'] = cpy_flags
+
             db_conn = mysql.connector.connect(**parameters)
             # Return MySQL connection object.
             return db_conn
         except mysql.connector.Error as err:
-            raise UtilError("Cannot connect to the {0} server.\n"
-                            "Error {1}".format(self.role, err.msg), err.errno)
+            raise UtilError(err.msg, err.errno)
+        except AttributeError as err:
+            raise UtilError(str(err))
 
     def disconnect(self):
         """Disconnect from the server.
@@ -974,12 +1133,18 @@ class Server(object):
         # If we are fetching all, we need to use a buffered
         if fetch:
             if raw:
-                cur = self.db_conn.cursor(
-                    cursor_class=mysql.connector.cursor.MySQLCursorBufferedRaw)
+                if mysql.connector.__version_info__ < (2, 0):
+                    cur = self.db_conn.cursor(buffered=True, raw=True)
+                else:
+                    cur = self.db_conn.cursor(
+                        cursor_class=MySQLUtilsCursorBufferedRaw)
             else:
                 cur = self.db_conn.cursor(buffered=True)
         else:
-            cur = self.db_conn.cursor(raw=True)
+            if mysql.connector.__version_info__ < (2, 0):
+                cur = self.db_conn.cursor(raw=True)
+            else:
+                cur = self.db_conn.cursor(cursor_class=MySQLUtilsCursorRaw)
 
         # Execute query, handling parameters.
         q_killer = None
@@ -1811,7 +1976,11 @@ class QueryKillerThread(threading.Thread):
             # kill the query.
             if not self._stop_event.is_set():
                 try:
-                    cur = self._connection.cursor(raw=True)
+                    if mysql.connector.__version_info__ < (2, 0):
+                        cur = self._connection.cursor(raw=True)
+                    else:
+                        cur = self._connection.cursor(
+                            cursor_class=MySQLUtilsCursorRaw)
 
                     # Get process information from threads table when available
                     # (for versions > 5.6.1), since it does not require a mutex
